!{\src2tex{textfont=tt}}
!!****f* ABINIT/ortho_phi
!! NAME
!! ortho_phi
!! 
!! FUNCTION
!! Re-compute and re-orthogonalize partial waves (Phi), pseudo partial waves (tPhi)
!! and projectors (tProj) according to the integration scheme "ctrap" from Abinit
!!
!! COPYRIGHT
!! Copyright (C) 1998-2004 ABINIT group (FJ, MT)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~ABINIT/Infos/copyright
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~ABINIT/Infos/contributors .
!! 
!! INPUTS
!!  pawarray
!!    %shapefunc(mesh_size)= Normalized shape function
!!  pshead
!!    %basis_size= Number of elements for the paw nl basis
!!    %lmax= Maximum value of l
!!    %mesh_size= Dimension of radial mesh for phi, tphi ...
!!    %orbitals(basis_size)= Quantum number l for each basis function
!!    %rad_step= Step corresponding to radial mesh
!!
!! SIDE EFFECTS
!!  pawps
!!    %phi(mesh_size,basis_size)= PAW atomic wavefunctions
!!                                on the radial grid
!!    %tphi(mesh_size,basis_size)= PAW atomic pseudo-wavefunctions
!!                                 on the radial grid
!!    %tproj(proj_meshsz,basis_size)= PAW projectors
!!                                 on the radial grid
!!
!! PARENTS
!!      atompaw2abinit
!!
!! CHILDREN
!!      ctrap
!!
!! SOURCE

 subroutine ortho_phi(pawarray,pawps,pshead)

 use defs_basis
 use defs_pawps

 implicit none

!Arguments ---------------------------------------------
!These types are defined in defs_pawps
 type(pawarray_type) :: pawarray
 type(pawps_type)    :: pawps
 type(pshead_type)   :: pshead

!Local variables ---------------------------------------
 integer, parameter :: option=1

 integer :: ib,ib1,ib2,il,msize
 real(dp) :: f2l,intg1,intg2
 integer :: iref(2,4),nref(4)
 real(dp), allocatable :: ff(:)

! *************************************************************************

 if (option==0) return
 if (option/=1.and.option/=2) stop 'Bug in ortho_phi !'

 msize=pshead%mesh_size;allocate(ff(msize))

!Recompute tproj from tphi
!-------------------------
 if (option==2) then
  do ib=1,pshead%basis_size
!  Compute <tPhi_i|k|tPhi_i>
   ff(1:msize)=pawarray%shapefunc(1:msize)*pawps%tphi(1:msize,ib)**2
   call ctrap(msize,ff,pshead%rad_step,intg1)
! Compute k(r).tPhi_i(r)/<tPhi_i|k|tPhi_i>
   pawps%tproj(1:msize,ib)=pawarray%shapefunc(1:msize)*pawps%tphi(1:msize,ib)/intg1
  enddo
 end if

!Search for links between angular momenta and partial waves
 nref=0
 do il=1,pshead%lmax+1
  do ib=1,pshead%basis_size
   if (pshead%orbitals(ib)==(il-1)) then
    if (nref(il)==0) then
     nref(il)=1;iref(1,il)=ib
    else
     nref(il)=nref(il)+1
     if (nref(il)>2) then
      write(6,'(/,a,/,a,i1,a,/,a)') 'Info:',&
&           '  Too much partial waves corrresponding to l=',il,' !',&
&           '  Program stopped.'
      stop
     endif
     iref(nref(il),il)=ib
    endif
   endif
  enddo
 enddo

!Re-orthogonalize Phi, tPhi and tProj
!for each l with 2 partial waves
!------------------------------------
 do il=1,pshead%lmax+1
  if (nref(il)==2) then
   ib1=iref(1,il);ib2=iref(2,il)
!  Compute sqrt(1-<tPhi_2l|tPhi_1l><tPhi_1l|tProj_2l>)
   ff(1:msize)=pawps%tphi(1:msize,ib2)*pawps%tproj(1:msize,ib1)
   call ctrap(msize,ff,pshead%rad_step,intg1)
   ff(1:msize)=pawps%tphi(1:msize,ib1)*pawps%tproj(1:msize,ib2)
   call ctrap(msize,ff,pshead%rad_step,intg2)
   f2l=1.d0/sqrt(1.d0-intg1*intg2)
!  Compute <tProj_1l|tPhi_2l>
   ff(1:msize)=pawps%tproj(1:msize,ib1)*pawps%tphi(1:msize,ib2)
   call ctrap(msize,ff,pshead%rad_step,intg1)
!  Compute <tPhi_1l|tProj_2l>
   ff(1:msize)=pawps%tphi(1:msize,ib1)*pawps%tproj(1:msize,ib2)
   call ctrap(msize,ff,pshead%rad_step,intg2)
!  Compute Phi_2l, tPhi_2l, tProj_2l
   pawps%phi  (1:msize,ib2)=f2l*(pawps%phi  (1:msize,ib2)-pawps%phi  (1:msize,ib1)*intg1)
   pawps%tphi (1:msize,ib2)=f2l*(pawps%tphi (1:msize,ib2)-pawps%tphi (1:msize,ib1)*intg1)
   pawps%tproj(1:msize,ib2)=f2l*(pawps%tproj(1:msize,ib2)-pawps%tproj(1:msize,ib1)*intg2)
  endif
 enddo

 deallocate(ff)

 end subroutine
!!***
